@@ -12,6 +12,9 @@ Acf <- function(x, lag.max = NULL,
 
   acf.out <- stats::acf(x, plot=FALSE, lag.max=lag.max, type=type, na.action=na.action, demean=demean)
 
+  acf.out$tsp <- tsp(x)
+  acf.out$periods <- attributes(x)$msts
+
   if(nseries==1)
   {
     vname <- deparse(substitute(x))
@@ -97,7 +100,8 @@ seasonalaxis <- function(frequency, nlags, type, plot=TRUE)
           out2 <- 1:nlags
         else if(nlags <= 15 & type=="ccf")
           out2 <- (-nlags:nlags)
-        out <- pretty(out2)
+        if(!is.null(out2))
+          out <- pretty(out2)
       }
     }
     else if(frequency > 1 & ((type=="acf" & np >= 2L) | (type=="ccf" & np >= 1L)))
@@ -144,9 +148,38 @@ seasonalaxis <- function(frequency, nlags, type, plot=TRUE)
 }
 
 
-Pacf <- function (x, lag.max=NULL, plot=TRUE, na.action=na.contiguous, ...)
+Pacf <- function (x, lag.max=NULL, 
+                plot = TRUE, na.action = na.contiguous, demean=TRUE, ...)
 {
-  Acf(x, type="partial", lag.max=lag.max, plot=plot, na.action=na.action, ...)
+  object <- Acf(x, lag.max=lag.max, type="partial", na.action=na.action, demean=demean, plot=FALSE)
+  object$series <- deparse(substitute(x))
+
+  # Plot if required
+  if(plot)
+  {
+    nlags <- dim(object$lag)[1]
+    plot.out <- object
+    # Check if there is a ylim input
+    input_list <- as.list(substitute(list(...)))
+    ylimarg <- is.element("ylim",names(input_list))
+    if(ylimarg)
+      plot(plot.out, xaxt="n", ...)
+    else
+    {
+      ylim <- c(-1, 1) * 3/sqrt(length(x))
+      ylim <- range(ylim, plot.out$acf)
+      plot(plot.out, ylim=ylim, xaxt="n", ...)
+    }
+    # Make nice horizontal axis
+    if(is.element("msts", class(x)))
+      seasonalaxis(attributes(x)$msts, nlags, type="acf")
+    else
+      seasonalaxis(frequency(x), nlags, type="acf")
+    return(invisible(object))
+  }
+  else
+    return(object)
+
 }
 
 Ccf <- function (x, y, lag.max=NULL, type=c("correlation","covariance"),
