@@ -9,6 +9,8 @@ ets <- function(y, model="ZZZ", damped=NULL,
   opt.crit <- match.arg(opt.crit)
   bounds <- match.arg(bounds)
   ic <- match.arg(ic)
+  
+  seriesname <- deparse(substitute(y))
 
   if(any(class(y) %in% c("data.frame","list","matrix","mts")))
     stop("y should be a univariate time series")
@@ -88,12 +90,11 @@ ets <- function(y, model="ZZZ", damped=NULL,
       model$residuals <- ts(e$e,frequency=tsp.y[3],start=tsp.y[1])
       model$sigma2 <- mean(model$residuals^2,na.rm=TRUE)
       model$x <- orig.y
+      model$series <- seriesname
       if(!is.null(lambda))
       {
-        model$fitted <- InvBoxCox(model$fitted,lambda)
-        if(biasadj){
-          model$fitted <- InvBoxCoxf(x = model$fitted, fvar = var(model$residuals), lambda = lambda)
-        }
+        model$fitted <- InvBoxCox(model$fitted, lambda, biasadj, var(model$residuals))
+        attr(lambda, "biasadj") <- biasadj
       }
       model$lambda <- lambda
 
@@ -240,20 +241,19 @@ ets <- function(y, model="ZZZ", damped=NULL,
 
   model$m <- m
   model$method <- paste("ETS(",best.e,",",best.t,ifelse(best.d,"d",""),",",best.s,")",sep="")
+  model$series <- seriesname
   model$components <- c(best.e,best.t,best.s,best.d)
   model$call <- match.call()
   model$initstate <- model$states[1,]
   model$sigma2 <- mean(model$residuals^2,na.rm=TRUE)
   model$x <- orig.y
-  model$lambda <- lambda
   if(!is.null(lambda))
   {
-    model$fitted <- InvBoxCox(model$fitted,lambda)
-    if(biasadj){
-      model$fitted <- InvBoxCoxf(x = model$fitted, fvar = var(model$residuals), lambda = lambda)
-    }
+    model$fitted <- InvBoxCox(model$fitted,lambda, biasadj, var(model$residuals))
+    attr(lambda, "biasadj") <- biasadj
   }
-
+  
+  model$lambda <- lambda
   #model$call$data <- dataname
 
   return(structure(model,class="ets"))
@@ -375,7 +375,7 @@ etsmodel <- function(y, errortype, trendtype, seasontype, damped,
   #         seasontype=seasontype, damped=damped, par.noopt=par.noopt, lowerb=lower, upperb=upper,
   #         opt.crit=opt.crit, nmse=nmse, bounds=bounds, m=m,pnames=names(par),pnames2=names(par.noopt))
 
-  #     func <- .Call("etsGetTargetFunctionRmalschainsPtr", package="forecast")
+  #     func <- .Call("etsGetTargetFunctionRmalschainsPtr", PACKAGE="forecast")
 
   #   }
 
@@ -406,7 +406,7 @@ etsmodel <- function(y, errortype, trendtype, seasontype, damped,
 #        seasontype=seasontype, damped=damped, par.noopt=par.noopt, lowerb=lower, upperb=upper,
 #        opt.crit=opt.crit, nmse=nmse, bounds=bounds, m=m,pnames=names(par),pnames2=names(par.noopt))
 #
-#    func <- .Call("etsGetTargetFunctionRdonlp2Ptr", package="forecast")
+#    func <- .Call("etsGetTargetFunctionRdonlp2Ptr", PACKAGE="forecast")
 #
 #    myBounds <- getNewBounds(par, lower, upper, nstate)
 #
@@ -423,7 +423,7 @@ etsmodel <- function(y, errortype, trendtype, seasontype, damped,
         opt.crit=opt.crit, nmse=as.integer(nmse), bounds=bounds, m=m,pnames=names(par),pnames2=names(par.noopt))
 
     fred <- .Call("etsNelderMead", par, env, -Inf,
-        sqrt(.Machine$double.eps), 1.0, 0.5, 2.0, trace, maxit, package="forecast")
+        sqrt(.Machine$double.eps), 1.0, 0.5, 2.0, trace, maxit, PACKAGE="forecast")
 
     fit.par <- fred$par
 
@@ -583,7 +583,7 @@ etsTargetFunctionInit <- function(par,y,nstate,errortype,trendtype,seasontype,da
       opt.crit=opt.crit, nmse=as.integer(nmse), bounds=bounds, m=m,
       optAlpha, optBeta, optGamma, optPhi,
       givenAlpha, givenBeta, givenGamma, givenPhi,
-      alpha, beta, gamma, phi, env, package="forecast")
+      alpha, beta, gamma, phi, env, PACKAGE="forecast")
   res
 }
 
@@ -1032,7 +1032,12 @@ fitted.ets <- function(object, h=1, ...){
 
 logLik.ets <- function(object,...)
 {
-  structure(object$loglik,df=length(object$par),class="logLik")
+  structure(object$loglik,df=length(object$par)+1,class="logLik")
+}
+
+nobs.ets <- function(object, ...)
+{
+  length(object$x)
 }
 
 is.ets <- function(x){
