@@ -1,13 +1,23 @@
 forecast.ets <- function(object, h=ifelse(object$m>1, 2*object$m, 10),
-  level=c(80,95), fan=FALSE, simulate=FALSE, bootstrap=FALSE, npaths=5000, PI=TRUE, 
-  lambda=object$lambda, biasadj=FALSE, ...)
+  level=c(80,95), fan=FALSE, simulate=FALSE, bootstrap=FALSE, npaths=5000, PI=TRUE,
+  lambda=object$lambda, biasadj=NULL, ...)
 {
   # Check inputs
   #if(h>2000 | h<=0)
   if(h <= 0)
     stop("Forecast horizon out of bounds")
-  if(is.null(lambda))
+  if(is.null(lambda)){
     biasadj <- FALSE
+  }
+  else{
+    if(is.null(biasadj)){
+      biasadj <- attr(lambda, "biasadj")
+    }
+    if(!is.logical(biasadj)){
+      warning("biasadj information not found, defaulting to FALSE.")
+      biasadj <- FALSE
+    }
+  }
   if(!PI & !biasadj)
   {
     simulate <- bootstrap <- fan <- FALSE
@@ -24,6 +34,8 @@ forecast.ets <- function(object, h=ifelse(object$m>1, 2*object$m, 10),
     else if(min(level) < 0 | max(level) > 99.99)
       stop("Confidence limit out of range")
   }
+  # Order levels
+  level <- sort(level)
 
   n <- length(object$x)
   damped <- as.logical(object$components[4])
@@ -50,6 +62,7 @@ forecast.ets <- function(object, h=ifelse(object$m>1, 2*object$m, 10),
     if(!is.null(f$var))
     {
       out$lower <- out$upper <- ts(matrix(NA,ncol=length(level),nrow=h))
+      colnames(out$lower) <- colnames(out$upper) <- paste(level,"%",sep="")
       for(i in 1:length(level))
       {
         marg.error <- sqrt(f$var) * abs(qnorm((100-level[i])/200))
@@ -66,22 +79,25 @@ forecast.ets <- function(object, h=ifelse(object$m>1, 2*object$m, 10),
     }
     else if(PI)
       warning("No prediction intervals for this model")
-    else if(biasadj)
+    else if(any(biasadj))
       warning("No bias adjustment possible")
   }
-	
+
   out$fitted <- fitted(object)
   out$method <- object$method
+  if(!is.null(object$series)){
+    out$series <- object$series
+  }
+  else{
+    out$series <- object$call$y
+  }
   out$residuals <- residuals(object)
 
   if(!is.null(lambda))
   {
 	  #out$x <- InvBoxCox(object$x,lambda)
 	  #out$fitted <- InvBoxCox(out$fitted,lambda)
-    out$mean <- InvBoxCox(out$mean,lambda)
-    if(biasadj){
-      out$mean <- InvBoxCoxf(x = out, lambda = lambda)
-    }
+    out$mean <- InvBoxCox(out$mean,lambda, biasadj, out)
 	  if(PI)  # PI = TRUE
 	  {
  		  out$lower <- InvBoxCox(out$lower,lambda)
@@ -90,7 +106,7 @@ forecast.ets <- function(object, h=ifelse(object$m>1, 2*object$m, 10),
   }
   if(!PI)
     out$lower <- out$upper <- out$level <- NULL
-	
+
   return(structure(out,class="forecast"))
 }
 
