@@ -1,6 +1,6 @@
 ### Time series graphics and transformations
 
-tsdisplay <- function(x,plot.type=c("partial", "scatter", "spectrum"),points=TRUE,ci.type=c("white", "ma"),
+tsdisplay <- function(x,plot.type=c("partial", "histogram","scatter", "spectrum"),points=TRUE,ci.type=c("white", "ma"),
                 lag.max, na.action=na.contiguous, main=NULL,xlab="",ylab="",
                 pch=1,cex=0.5, ...)
 
@@ -43,6 +43,10 @@ tsdisplay <- function(x,plot.type=c("partial", "scatter", "spectrum"),points=TRU
   }
   else if(plot.type == "spectrum")
     spec.ar(x,main="",na.action=na.action)
+  else if(plot.type=="histogram")
+  {
+    graphics::hist(x, breaks="FD", main="", xlab=main)
+  }
   else
     plot(junk2,ylim=ylim,xlim=c(1,lag.max),ylab="PACF",main="",...)
   par(def.par)
