@@ -3,10 +3,9 @@
 # lag=1 corresponds to standard random walk (i.e., naive forecast)
 # lag=m corresponds to seasonal naive method
 
-lagwalk <- function(y, lag=1, h=10, drift=FALSE, 
+lagwalk <- function(y, lag=1, h=10, drift=FALSE,
   level=c(80,95), fan=FALSE, lambda=NULL, biasadj=FALSE)
 {
-  xname <- deparse(substitute(y))
   n <- length(y)
   m <- frequency(y)
   nn <- 1:h
@@ -35,7 +34,10 @@ lagwalk <- function(y, lag=1, h=10, drift=FALSE,
   fits <- ts(c(rep(NA,lag),head(y,-lag)) + b, start=tsp(y)[1], frequency=m)
   res <- y - fits
   fullperiods <- (h-1)/lag+1
-  steps <- rep(1:fullperiods, rep(m,fullperiods))[1:h]
+  if(lag==1)
+    steps <- 1:h
+  else
+    steps <- rep(1:fullperiods, rep(m,fullperiods))[1:h]
   f <- rep(tail(y,lag), fullperiods)[1:h] + steps*b
   mse <- mean(res^2, na.rm=TRUE)
   se  <- sqrt(mse*steps  + (steps*b.se)^2)
@@ -64,17 +66,15 @@ lagwalk <- function(y, lag=1, h=10, drift=FALSE,
   if(!is.null(lambda))
   {
     y <- origy
-    fcast <- InvBoxCox(fcast,lambda)
-    if(biasadj){
-      fcast$mean <- InvBoxCoxf(x = list(level = level, mean = fcast, upper = upper, lower = lower), lambda = lambda) 
-    }
+    fcast <- InvBoxCox(fcast, lambda, biasadj, list(level = level, upper = upper, lower = lower))
     fits <- InvBoxCox(fits,lambda)
     upper <- InvBoxCox(upper,lambda)
     lower <- InvBoxCox(lower,lambda)
   }
 
-  out <- list(method=method,level=level,x=y,xname=xname,mean=fcast,lower=lower,upper=upper,
-      model=list(drift=b,drift.se=b.se,sd=s), fitted = fits, residuals = res, lambda=lambda)
+  out <- list(method=method,level=level,x=y,mean=fcast,lower=lower,upper=upper,
+      model=structure(list(includedrift=drift,drift=b,drift.se=b.se,sd=s),class='naive'),
+      fitted = fits, residuals = res, lambda=lambda)
   out$model$call <- match.call()
 
   return(structure(out,class="forecast"))
@@ -84,10 +84,10 @@ lagwalk <- function(y, lag=1, h=10, drift=FALSE,
 # Random walk
 rwf <- function(y,h=10,drift=FALSE,level=c(80,95),fan=FALSE,lambda=NULL,biasadj=FALSE,x=y)
 {
-  fc <- lagwalk(x, lag=1, h=h, drift=drift, level=level, fan=fan, 
+  fc <- lagwalk(x, lag=1, h=h, drift=drift, level=level, fan=fan,
     lambda=lambda, biasadj=biasadj)
-  fc$xname <- deparse(substitute(y))
   fc$model$call <- match.call()
+  fc$series <- deparse(substitute(y))
 
   if(drift)
     fc$method <- "Random walk with drift"
@@ -95,7 +95,7 @@ rwf <- function(y,h=10,drift=FALSE,level=c(80,95),fan=FALSE,lambda=NULL,biasadj=
     fc$method <- "Random walk"
   return(fc)
 }
-  
+
 # naive <- function(x,h=10,level=c(80,95),fan=FALSE, lambda=NULL)
 # {
 #     fc <- forecast(Arima(x,order=c(0,1,0),lambda=lambda),h,level=level,fan=fan)
@@ -109,8 +109,8 @@ rwf <- function(y,h=10,drift=FALSE,level=c(80,95),fan=FALSE,lambda=NULL,biasadj=
 naive <- function(y,h=10,level=c(80,95),fan=FALSE, lambda=NULL, biasadj=FALSE,x=y)
 {
   fc <- rwf(x, h=h, level=level, fan=fan, lambda=lambda, drift=FALSE, biasadj=biasadj)
-  fc$xname <- deparse(substitute(y))
   fc$model$call <- match.call()
+  fc$series <- deparse(substitute(y))
   fc$method <- "Naive method"
   return(fc)
 }
@@ -129,10 +129,18 @@ naive <- function(y,h=10,level=c(80,95),fan=FALSE, lambda=NULL, biasadj=FALSE,x=
 
 snaive <- function(y, h=2*frequency(x), level=c(80,95), fan=FALSE, lambda=NULL, biasadj=FALSE,x=y)
 {
-  fc <- lagwalk(x, lag=frequency(x), h=h, drift=FALSE, level=level, fan=fan, 
+  fc <- lagwalk(x, lag=frequency(x), h=h, drift=FALSE, level=level, fan=fan,
     lambda=lambda, biasadj=biasadj)
-  fc$xname <- deparse(substitute(y))
   fc$model$call <- match.call()
+  fc$series <- deparse(substitute(y))
   fc$method <- "Seasonal naive method"
   return(fc)
 }
+
+print.naive <- function(x, ...)
+{
+  cat(paste("Call:", deparse(x$call), "\n\n"))
+  if(x$includedrift)
+    cat(paste("Drift: ",round(x$drift,4),"  (se ",round(x$drift.se,4), ")\n", sep=""))
+  cat(paste("Residual sd:", round(x$sd,4), "\n"))
+}
