@@ -3,11 +3,13 @@
 
 tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 	seasonal.periods=NULL, use.arma.errors=TRUE, use.parallel=length(y)>1000, num.cores=2,
-	bc.lower=0, bc.upper=1, model=NULL, ...)
+	bc.lower=0, bc.upper=1, biasadj=FALSE, model=NULL, ...)
 {
   if (any(class(y) %in% c("data.frame", "list", "matrix", "mts")))
     stop("y should be a univariate time series")
 
+  seriesname <- deparse(substitute(y))
+  
   origy <- y
 
   # Get seasonal periods
@@ -64,7 +66,7 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 	non.seasonal.model <- bats(as.numeric(y), use.box.cox=use.box.cox, use.trend=use.trend,
 	                           use.damped.trend=use.damped.trend, use.arma.errors=use.arma.errors,
 	                           use.parallel=use.parallel, num.cores=num.cores,
-	                           bc.lower=bc.lower, bc.upper=bc.upper, ...)
+	                           bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj, ...)
 
 
   # If non-seasonal data, return the non-seasonal model
@@ -117,7 +119,7 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 	best.model <- try(fitSpecificTBATS(y, use.box.cox = model.params[1], use.beta = model.params[2],
 	                      use.damping = model.params[3], seasonal.periods = seasonal.periods,
 	                      k.vector = k.vector, init.box.cox=init.box.cox,
-	                      bc.lower=bc.lower, bc.upper=bc.upper), silent=TRUE)
+	                      bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj), silent=TRUE)
   if(is.element("try-error",class(best.model)))
 	  best.model <- list(AIC=Inf)
 
@@ -157,7 +159,7 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 					new.model <- try(fitSpecificTBATS(y, use.box.cox = model.params[1], use.beta = model.params[2],
 					                                  use.damping = model.params[3], seasonal.periods = seasonal.periods,
 					                                  k.vector = k.vector, init.box.cox=init.box.cox,
-					                                  bc.lower=bc.lower, bc.upper=bc.upper),
+					                                  bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 						silent=TRUE)
 					if(is.element("try-error",class(new.model)))
 						new.model <- list(AIC=Inf)
@@ -191,7 +193,7 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 					                              box.cox=model.params[1], trend = model.params[2],
 					                              damping = model.params[3], seasonal.periods = seasonal.periods,
 					                              k.control.matrix=k.control.array, init.box.cox=init.box.cox,
-					                              bc.lower=bc.lower, bc.upper=bc.upper)
+					                              bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj)
 					up.model <- models.list[[1]]
 					level.model <- models.list[[3]]
 					down.model <- models.list[[2]]
@@ -199,19 +201,19 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 					up.model <- try(fitSpecificTBATS(y, use.box.cox = model.params[1],
 					                                 use.beta = model.params[2], use.damping = model.params[3],
 					                                 seasonal.periods = seasonal.periods, k.vector = step.up.k,
-					                                 init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+					                                 init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 						silent=TRUE)
   				if(is.element("try-error",class(up.model)))
 						up.model <- list(AIC=Inf)
 					level.model <- try(fitSpecificTBATS(y, use.box.cox = model.params[1], use.beta = model.params[2],
 					                                    use.damping = model.params[3], seasonal.periods = seasonal.periods,
-					                                    k.vector = k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+					                                    k.vector = k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 						silent=TRUE)
   				if(is.element("try-error",class(level.model)))
 						level.model <- list(AIC=Inf)
 					down.model <- try(fitSpecificTBATS(y, use.box.cox = model.params[1], use.beta = model.params[2],
 					                                   use.damping = model.params[3], seasonal.periods = seasonal.periods,
-					                                   k.vector = step.down.k, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+					                                   k.vector = step.down.k, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 						silent=TRUE)
   				if(is.element("try-error",class(down.model)))
 						down.model <- list(AIC=Inf)
@@ -226,7 +228,7 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 						k.vector[i] <- k.vector[i]-1
 						down.model <- try(fitSpecificTBATS(y=y, use.box.cox=model.params[1], use.beta=model.params[2],
 						                                   use.damping=model.params[3], seasonal.periods=seasonal.periods,
-						                                   k.vector=k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+						                                   k.vector=k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
   						silent=TRUE)
 	  				if(is.element("try-error",class(down.model)))
 							down.model <- list(AIC=Inf)
@@ -251,7 +253,7 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 					k.vector[i] <- 7
 					repeat {
 						k.vector[i] <- k.vector[i]+1
-						up.model <- try(fitSpecificTBATS(y, model.params[1], model.params[2], model.params[3], seasonal.periods, k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+						up.model <- try(fitSpecificTBATS(y, model.params[1], model.params[2], model.params[3], seasonal.periods, k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
   						silent=TRUE)
     				if(is.element("try-error",class(up.model)))
 	  					up.model <- list(AIC=Inf)
@@ -302,7 +304,7 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 				}
 			}
 		}
-		models.list <- clusterApplyLB(clus, c(1:nrow(control.array)), parFilterTBATSSpecifics, y=y, control.array=control.array, model.params=model.params, seasonal.periods=seasonal.periods, k.vector=k.vector, use.arma.errors=use.arma.errors, aux.model=aux.model, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, ...)
+		models.list <- clusterApplyLB(clus, c(1:nrow(control.array)), parFilterTBATSSpecifics, y=y, control.array=control.array, model.params=model.params, seasonal.periods=seasonal.periods, k.vector=k.vector, use.arma.errors=use.arma.errors, aux.model=aux.model, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj, ...)
 		stopCluster(clus)
 		##Choose the best model
 		####Get the AICs
@@ -321,9 +323,9 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 			for(trend in use.trend) {
 				for(damping in use.damped.trend) {
 					if(all((model.params == c(box.cox, trend, damping)))) {
-						new.model <- filterTBATSSpecifics(y, box.cox, trend, damping, seasonal.periods, k.vector, use.arma.errors, aux.model=aux.model, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, ...)
+						new.model <- filterTBATSSpecifics(y, box.cox, trend, damping, seasonal.periods, k.vector, use.arma.errors, aux.model=aux.model, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj, ...)
 					} else if(!((trend == FALSE) & (damping == TRUE))) {
-						new.model <- filterTBATSSpecifics(y, box.cox, trend, damping, seasonal.periods, k.vector, use.arma.errors, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, ...)
+						new.model <- filterTBATSSpecifics(y, box.cox, trend, damping, seasonal.periods, k.vector, use.arma.errors, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj, ...)
 					}
 					if(new.model$AIC < best.model$AIC) {
 						best.model <- new.model
@@ -337,17 +339,18 @@ tbats <- function(y, use.box.cox=NULL, use.trend=NULL, use.damped.trend=NULL,
 	best.model$call <- match.call()
 	attributes(best.model$fitted.values) <- attributes(best.model$errors) <- attributes(origy)
 	best.model$y <- origy
+	best.model$series <- seriesname
 
 	return(best.model)
 }
 
 ######################################################################################################################################
-parFilterTBATSSpecifics <- function(control.number, y, control.array, model.params, seasonal.periods, k.vector, use.arma.errors, aux.model=NULL, init.box.cox=NULL, bc.lower=0, bc.upper=1, ...) {
+parFilterTBATSSpecifics <- function(control.number, y, control.array, model.params, seasonal.periods, k.vector, use.arma.errors, aux.model=NULL, init.box.cox=NULL, bc.lower=0, bc.upper=1, biasadj=FALSE, ...) {
 	box.cox <- control.array[control.number, 1]
 	trend <- control.array[control.number, 2]
 	damping <- control.array[control.number, 3]
 	if(!all((model.params == c(box.cox, trend, damping)))) {
-		first.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+		first.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 			silent=TRUE)
 	} else {
 		first.model <- aux.model
@@ -374,7 +377,7 @@ parFilterTBATSSpecifics <- function(control.number, y, control.array, model.para
 				}
 				starting.params <- first.model$parameters
 
-				second.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, ar.coefs=ar.coefs, ma.coefs=ma.coefs, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+				second.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, ar.coefs=ar.coefs, ma.coefs=ma.coefs, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 					silent=TRUE)
 				if(is.element("try-error", class(second.model)))
   				second.model <- list(AIC=Inf)
@@ -395,19 +398,19 @@ parFilterTBATSSpecifics <- function(control.number, y, control.array, model.para
 }
 
 #################################################################################################
-parFitSpecificTBATS <- function(control.number, y, box.cox, trend, damping, seasonal.periods, k.control.matrix, init.box.cox=NULL, bc.lower=0, bc.upper=1) {
+parFitSpecificTBATS <- function(control.number, y, box.cox, trend, damping, seasonal.periods, k.control.matrix, init.box.cox=NULL, bc.lower=0, bc.upper=1, biasadj=FALSE) {
 	k.vector <- k.control.matrix[control.number,]
 
-	model <- try(fitSpecificTBATS(y, use.box.cox = box.cox, use.beta = trend, use.damping = damping, seasonal.periods = seasonal.periods, k.vector = k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+	model <- try(fitSpecificTBATS(y, use.box.cox = box.cox, use.beta = trend, use.damping = damping, seasonal.periods = seasonal.periods, k.vector = k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 		silent=TRUE)
 	if(is.element("try-error", class(model)))
 		model <- list(AIC=Inf)
 	return(model)
 }
 
-filterTBATSSpecifics <- function(y, box.cox, trend, damping, seasonal.periods, k.vector, use.arma.errors, aux.model=NULL, init.box.cox=NULL, bc.lower=0, bc.upper=1, ...) {
+filterTBATSSpecifics <- function(y, box.cox, trend, damping, seasonal.periods, k.vector, use.arma.errors, aux.model=NULL, init.box.cox=NULL, bc.lower=0, bc.upper=1, biasadj=FALSE, ...) {
 	if(is.null(aux.model)) {
-		first.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+		first.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 			silent=TRUE)
 	} else {
 		first.model <- aux.model
@@ -433,7 +436,7 @@ filterTBATSSpecifics <- function(y, box.cox, trend, damping, seasonal.periods, k
 				}
 				starting.params <- first.model$parameters
 
-				second.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, ar.coefs=ar.coefs, ma.coefs=ma.coefs, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper),
+				second.model <- try(fitSpecificTBATS(y, use.box.cox=box.cox, use.beta=trend, use.damping=damping, seasonal.periods=seasonal.periods, k.vector=k.vector, ar.coefs=ar.coefs, ma.coefs=ma.coefs, init.box.cox=init.box.cox, bc.lower=bc.lower, bc.upper=bc.upper, biasadj=biasadj),
 					silent=TRUE)
 				if(is.element("try-error",class(second.model)))
 					second.model <- list(AIC=Inf)
